// Generated by CoffeeScript 2.6.1
(function() {
  var LazyValue, isFunction, select,
    splice = [].splice;

  ({isFunction} = require("../helpers"));

  LazyValue = class LazyValue {
    constructor(fetch) {
      var $;
      if (!isFunction(fetch)) {
        throw new Error("It's not a function!");
      }
      this.fetch = fetch;
      this.state = "start"; // start, waiting, or ready
      this.cbs = [];
      $ = this;
      $.get = function(...args) {
        var cb, ref;
        ref = args, [...args] = ref, [cb] = splice.call(args, -1);
        switch ($.state) {
          case "start":
            $.state = "waiting";
            $.cbs.push(cb);
            setTimeout(function() {
              return $.fetch(function(err, data) {
                var results;
                $.state = "ready";
                $.data = data;
                $.err = err;
                results = [];
                while ($.cbs.length) {
                  results.push($.cbs.shift()(err, data));
                }
                return results;
              });
            }, 0);
            break;
          case "waiting":
            $.cbs.push(cb);
            break;
          case "ready":
            cb($.err, $.data);
            break;
          default:
            throw new Error(`Unknown state: ${$.state}`);
        }
        return $;
      };
    }

    get(...args) {
      var cb, ref;
      ref = args, [...args] = ref, [cb] = splice.call(args, -1);
      if (cb === void 0) {
        cb = function(err, value) {};
      }
    }

    // the method definition will be overwritten by constructor
    select(path, cb) {
      return select(this, path, cb);
    }

  };

  select = function(obj, path, cb) {
    var ref;
    if ((obj != null ? (ref = obj.constructor) != null ? ref.name : void 0 : void 0) === "LazyValue") {
      obj.get(function(err, val) {
        if (err) {
          return cb(err);
        }
        return select(val, path, cb);
      });
    } else if (path.length === 0) {
      cb(null, obj);
    } else {
      (function(field) {
        var e;
        try {
          return select(obj[field], path, cb);
        } catch (error) {
          e = error;
          return cb(e);
        }
      })(path.shift());
    }
    return void 0;
  };

  LazyValue.select = select;

  LazyValue.doc = `# \`value = new LazyValue( fetch_fn )\` creates a read only value initialized
# by an async \`fetch_fn\`. The value is read by \`value.get( cb )\`.  E.g.:
#
#   > var val = new LazyValue(function(cb){fs.readFile('/etc/passwd', cb);});
#   > val.get( function( err, data ){ console.log(data.toString() ) });
#
# There is also a helper method \`select\`:
# \`select(obj, path, cb)\` a field \`obj.path\` in \`obj\` which may contain
# \`LazyValue\`s.
# The callback \`cb(err,val)\` is called once value \`val\` of \`obj.path\`
# is retrived or error occurs.`;

  module.exports = LazyValue;

}).call(this);
