// Generated by CoffeeScript 2.6.1
(function() {
  var LazyValue, Maker, flatten, isArray, isFunction, isString, map, normalizeDeps, product, semaphore, topoOrder,
    splice = [].splice;

  product = require("../product");

  LazyValue = require("../LazyValue");

  map = require("../map");

  semaphore = require("../semaphore");

  ({flatten, isArray, isFunction, isString} = require("../helpers"));

  normalizeDeps = function(deps) {
    var m;
    m = [].concat(deps).reduce(function(m, x) {
      if (x != null) {
        m[x] = x;
      }
      return m;
    }, {});
    return Object.keys(m).sort();
  };

  topoOrder = function(spec) {
    var deps, inDegree, j, len, ref, result, starts, x, y;
    inDegree = Object.keys(spec).reduce(function(inDegree, v) {
      var j, len, ref, x;
      ref = spec[v].deps;
      for (j = 0, len = ref.length; j < len; j++) {
        x = ref[j];
        inDegree[x] = inDegree[x] ? inDegree[x] + 1 : 1;
      }
      return inDegree;
    }, {});
    starts = Object.keys(spec).filter(function(x) {
      return !inDegree[x];
    });
    result = [];
    while (x = starts.pop()) {
      result.push(x);
      deps = (ref = spec[x].deps) != null ? ref : [];
      for (j = 0, len = deps.length; j < len; j++) {
        y = deps[j];
        inDegree[y]--;
        if (inDegree[y] === 0) {
          starts.push(y);
        }
      }
    }
    return result;
  };

  Maker = class Maker {
    constructor(spec, parallel = 10) {
      var $, _defaultValue, _getFn, sem, v, val;
      $ = this;
      sem = semaphore(parallel);
      _defaultValue = function(deps, target) {
        return function(o, cb) {
          console.log(` simulating '${target}' ...`);
          return cb(null, `${target}[${Object.keys(o).sort()}]`);
        };
      };
      _getFn = function(target) {
        if (!spec[target]) {
          throw Error(`Unknown target: '${target}'`);
        }
        return function(_token, cb) {
          return spec[target]._value.get(cb);
        };
      };
      spec = Object.keys(spec).reduce(function(res, target) {
        var deps, ref, ref1, ref2;
        switch (false) {
          case !(isString(spec[target]) || isArray(spec[target])):
            deps = normalizeDeps(spec[target]);
            res[target] = {
              deps: deps,
              value: _defaultValue(deps, target)
            };
            break;
          case !isFunction(spec[target]):
            res[target] = {
              deps: [],
              value: spec[target]
            };
            break;
          default:
            deps = normalizeDeps((ref = spec[target]) != null ? ref.deps : void 0);
            res[target] = {
              deps: deps,
              value: (ref1 = (ref2 = spec[target]) != null ? ref2.value : void 0) != null ? ref1 : _defaultValue(deps, target)
            };
        }
        return res;
      }, {});
      if (topoOrder(spec).length !== Object.keys(spec).length) {
        throw Error("It is not a DAG!");
      }
      for (v in spec) {
        val = spec[v];
        (function(v, val) {
          return val._value = new LazyValue(function(cb) {
            return $.get(val.deps, function(err, data) {
              var o;
              if (err) {
                return cb(err);
              }
              o = [].concat(data).reduce(function(o, d, i) {
                o[val.deps[i]] = d;
                return o;
              }, {});
              return sem(val.value)(o, cb);
            });
          });
        })(v, val);
      }
      $.get = function(...targets) {
        var cb, fns, ref;
        ref = targets, [...targets] = ref, [cb] = splice.call(targets, -1);
        targets = flatten(targets);
        fns = targets.map(_getFn);
        if (targets.length === 1) {
          fns[0](targets[0], cb);
        } else {
          product(fns)(targets, cb);
        }
        return this;
      };
    }

  };

  Maker.doc = `#     maker = new Maker(spec)
#  constructs a DAG of 'targets'. Ex:
#     var Maker = require('functors/Maker'),
#       maker = new Maker({
#         a: (_,cb) => cb(null, 12),
#         b: {deps: 'a',
#             value: function(deps,cb) { cb(null, deps.a + 1); }}});
#      maker.get('a','b', (err, result) => console.log(result))
#     // should print: \`[12, 13]\``;

  module.exports = Maker;

}).call(this);
