// Generated by CoffeeScript 2.6.1
(function() {
  var flatten, isArray, isEmpty, isFunction, isNumber, isString, refrain, withContinuation,
    splice = [].splice;

  isArray = function(x) {
    return Array.isArray(x);
  };

  flatten = function(args) {
    return (function(res) {
      var i, j, len, len1, x, y;
      for (i = 0, len = args.length; i < len; i++) {
        x = args[i];
        if (isArray(x)) {
          for (j = 0, len1 = x.length; j < len1; j++) {
            y = x[j];
            res.push(y);
          }
        } else {
          res.push(x);
        }
      }
      return res;
    })([]);
  };

  isString = function(x) {
    return (typeof x === "string") || (x instanceof String);
  };

  isNumber = function(x) {
    return (typeof x === "number") || (x instanceof Number);
  };

  isFunction = function(x) {
    return typeof x === "function";
  };

  isEmpty = function(x) {
    switch (typeof x) {
      case "string":
        return x.length <= 0;
      case "undefined":
        return true;
      case "object":
        return (!x) || Object.keys(x).length <= 0;
    }
    return false;
  };

  withContinuation = function(syncFn, context) {
    return function(...args) {
      var cb, data, e, err, ref;
      ref = args, [...args] = ref, [cb] = splice.call(args, -1);
      data = null;
      err = null;
      try {
        data = syncFn.apply(context, args);
      } catch (error) {
        e = error;
        err = e;
      }
      return cb(err, data);
    };
  };

  refrain = function(promise, cb) {
    return promise.then(function(data) {
      return cb(null, data);
    }).catch(function(err) {
      return cb(err != null ? err : Error("Broken Promise"));
    });
  };

  module.exports = {
    flatten: flatten,
    isArray: isArray,
    isString: isString,
    isFunction: isFunction,
    isEmpty: isEmpty,
    isNumber: isNumber,
    withContinuation: withContinuation,
    refrain: refrain,
    doc: `# Helper (synchronous) functions:
#  flatten: e.g., [[1,2],3,4] -> [1,2,3,4] 
#  isArray: e.g., [1,2,3] -> true
#  isNumber: e.g., 0 -> true 
#  isString: e.g., 123 -> false
#  isFunction: ...
#  isEmpty: e.g., {} -> true, [] -> true, ""-> true, but 0 -> false
#  withContinuation: (fn) -> (args...,cb) -> cb null, fn args...
#  refrain: try to turn a Promise into an 'async-by-continuation' function`
  };

}).call(this);
